unit BestTimes;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XPMan, ComCtrls;

const
  NB_BEST_TIMES = 10;

type
  TBestTimesForm = class(TForm)
    CategoryCb: TComboBox;
    ListView: TListView;
    XPManifest1: TXPManifest;
    Button1: TButton;
    Resetbtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CategoryCbChange(Sender: TObject);
    procedure ResetbtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure RefreshCategories;
  end;

  TBestTime = record
    Name: string[64];
    Date: string[10];
    Time: Integer;
  end;

  PTimes = ^TTimes;
  TTimes = record
    Width, Height, Mines: Integer;
    BestTimes: array[1..NB_BEST_TIMES] of TBestTime;
  end;

var
  BestTimesForm: TBestTimesForm;
  Times: TList;

procedure LoadTimes;
procedure SaveTimes;
function GetTimes(W, H, M: Integer; Create: Boolean = False): PTimes;
procedure InsertTime(Category: PTimes; Name, Date: string; Time: Integer);

implementation

{$R *.dfm}

//------------------------------------------------------------------------------

procedure LoadTimes;
var
  I: Integer;
  F: file of TTimes;
  PtInt: PTimes;
begin
  if Times <> nil then
  begin
    for I := 0 to Times.Count - 1 do
      Dispose(Times[I]);
    Times.Free;
  end;
  Times := TList.Create;
  if FileExists(ExtractFilePath(ParamStr(0)) + 'BestTimes.dat') then
  begin
    AssignFile(F, ExtractFilePath(ParamStr(0)) + 'BestTimes.dat');
    Reset(F);
    while not Eof(F) do
    begin
      New(PtInt);
      BlockRead(F, PtInt^, 1);
      Times.Add(PtInt);
    end;
    CloseFile(F);
  end;
end;

//------------------------------------------------------------------------------

procedure SaveTimes;
var
  F: file of TTimes;
begin
  AssignFile(F, ExtractFilePath(ParamStr(0)) + 'BestTimes.dat');
  Rewrite(F);
  while Times.Count > 0 do
  begin
    BlockWrite(F, PTimes(Times[0])^, 1);
    Dispose(Times[0]);
    Times.Delete(0);
  end;
  CloseFile(F);
  Times.Free;
  Times := nil;
end;

//------------------------------------------------------------------------------

function GetTimes(W, H, M: Integer; Create: Boolean = False): PTimes;
var
  I: Integer;
  Found: Boolean;
  PtInt: PTimes;
begin
  Result := nil;
  Found := False;
  for I := 0 to Times.Count - 1 do
    with PTimes(Times[I])^ do
      if (Width = W) and (Height = H) and (Mines = M) then
      begin
        Result := Times[I];
        Found := True;
        Break;
      end;
  if not Found and Create then
  begin
    New(PtInt);
    with PtInt^ do
    begin
      Width := W;
      Height := H;
      Mines := M;
      for I := 1 to 10 do
        with BestTimes[I] do
        begin
          Name := '';
          Date := '';
          Time := -1;
        end;
    end;
    Times.Add(Ptint);
    Result := PtInt;
  end;
end;

//------------------------------------------------------------------------------

procedure InsertTime(Category: PTimes; Name, Date: string; Time: Integer);
var
  I, J: Integer;
begin
  if Category <> nil then
  begin
    I := 1;
    while (Category.BestTimes[I].Time > -1)
      and (Category.BestTimes[I].Time < Time) and (I <= NB_BEST_TIMES) do
      Inc(I);
    if I <= NB_BEST_TIMES then
    begin
      for J := NB_BEST_TIMES downto I + 1 do
        Category.BestTimes[J] := Category.BestTimes[J - 1];
      Category.BestTimes[I].Name := Name;
      Category.BestTimes[I].Date := Date;
      Category.BestTimes[I].Time := Time;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TBestTimesForm.FormCreate(Sender: TObject);
begin
  ClientWidth := 362;
  ClientHeight := 249;
end;

//------------------------------------------------------------------------------

procedure TBestTimesForm.CategoryCbChange(Sender: TObject);
var
  I: Integer;
begin
  if CategoryCb.ItemIndex > -1 then
    with PTimes(Times[CategoryCb.ItemIndex])^ do
    begin
      ListView.Items.BeginUpdate;
      for I := 1 to NB_BEST_TIMES do
      begin
        ListView.Items[I - 1].SubItems[0] := BestTimes[I].Name;
        ListView.Items[I - 1].SubItems[1] := BestTimes[I].Date;
        if BestTimes[I].Time > -1 then
          ListView.Items[I - 1].SubItems[2] := IntToStr(BestTimes[I].Time) + ' sec'
        else
          ListView.Items[I - 1].SubItems[2] := '';
      end;
      ListView.Items.EndUpdate;
    end
  else
  begin
    ListView.Items.BeginUpdate;
    for I := 1 to NB_BEST_TIMES do
    begin
      ListView.Items[I - 1].SubItems[0] := '';
      ListView.Items[I - 1].SubItems[1] := '';
      ListView.Items[I - 1].SubItems[2] := '';
    end;
    ListView.Items.EndUpdate;
  end;
end;

//------------------------------------------------------------------------------

procedure TBestTimesForm.RefreshCategories;
var
  I: Integer;

  function NameOfCat(PtTimes: PTimes): string;
  begin
    with PtTimes^ do
      if (Width = 9) and (Height = 9) and (Mines = 10) then
        Result := 'Beginner (9 x 9 x 10)'
      else if (Width = 16) and (Height = 16) and (Mines = 40) then
        Result := 'Beginner (16 x 16 x 40)'
      else if (Width = 30) and (Height = 16) and (Mines = 99) then
        Result := 'Beginner (30 x 16 x 99)'
      else
        Result := '(' + IntToStr(Width) + ' x ' + IntToStr(Height) + ' x ' + IntToStr(Mines) + ')';
  end;

begin
  CategoryCb.Items.Clear;
  if Times <> nil then
    for I := 0 to Times.Count - 1 do
      CategoryCb.Items.Add(NameOfCat(Times[I]));
  if CategoryCb.Items.Count > 0 then
    CategoryCb.ItemIndex := 0
  else CategoryCb.ItemIndex := -1;
end;

//------------------------------------------------------------------------------

procedure TBestTimesForm.ResetbtnClick(Sender: TObject);
begin
  if MessageDlg('Deleting Best scores for ' + CategoryCb.Items[CategoryCb.ItemIndex] + ' ?',
    mtInformation,
    [mbOk, mbCancel], 0) = mrOk then
  begin
    Dispose(Times[CategoryCb.ItemIndex]);
    Times.Delete(CategoryCb.ItemIndex);
    RefreshCategories;
    CategoryCbChange(nil);
  end;
end;

//------------------------------------------------------------------------------

end.

